<?php



require_once 'config.php';
require_once 'Classes/Astro/MJD.php';
require_once 'Classes/Queries/CalQuery.php';
require_once 'Classes/Queries/DateRangeQuery.php';
require_once 'Classes/Model/ObservationHeader.php';
require_once GREEN_ZONE_DIR . 'Utils/GZ_Exception.php';



/**
 * @class ConeQuery
 *
 * This class is an algorithm for querying the observation table for all the observations
 * that are within search_radius of the position given by rajd and decjd in the ConeQueryParameters.
 * The search is further refined by the the filtering options such as project_id, date range etc.
 * After we have retreived all the observations
 **/
class ConeQuery
{
  // PUBLIC

  public function __construct()
  {
    $this->parameters = '';
    $this->sql = '';
    $this->cone_string = '';
    $this->observations = null;
    $this->database = null;
    $this->num_rows_truncated = 0;
  }
  
  public function SetParameters( $s_parameters ) { $this->parameters = $s_parameters; }
  
  public function SetDatabase( $s_database ) { $this->database = $s_database; }
  
  public function GetNumRowsTruncated() { return $this->num_rows_truncated; }

  public function Execute()
  {
    if( $this->parameters == '' )
      throw new GZ_Exception( 'attempted to perform query without parameters' );

    if( $this->database == null )
      throw new GZ_Exception( 'Database not set', 'ConeQuery::Execute' );

    $this->observations = Array();

    // if we are only looking at timing data
    //   see how many results we would get from the observations table
    //   get those results limited to MAX_ROWS
    //   store the number truncated
    // otherwise if we are only looking at search mode data
    //   see how many results we would get from the s70 table
    //   get those results limited to MAX_ROWS
    // otherwise
    //   see how many results we would get from the observations table
    //   see how many results we would get from the s70 table
    //   get the results from the observations table limited to MAX_ROWS/2
    //   get the results from the s70 table limited to MAX_ROWS/2

    if( $this->parameters->GetObsMode() == 'timing' )
    {
      $obs_count = $this->CountObservationResults();
      $this->GetObservationHeaders( 'observations', MAX_ROWS );
      if( $obs_count > MAX_ROWS )
        $this->num_rows_truncated = $obs_count - MAX_ROWS;
    }
    else if( $this->parameters->GetObsMode() == 'search' )
    {
      $s70_count = $this->CountObservationResults();
      $this->GetObservationHeaders( 's70', MAX_ROWS );
      if( $s70_count > MAX_ROWS )
        $this->num_rows_truncated = $s70_count - MAX_ROWS;
    }
    else
    {
      $obs_count = $this->CountObservationResults();
      $s70_count = $this->CountObservationResults();
      $this->GetObservationHeaders( 'observations', MAX_ROWS / 2 );
      $this->GetObservationHeaders( 's70', MAX_ROWS / 2 );
      if( $s70_count > (MAX_ROWS / 2) )
        $this->num_rows_truncated += $s70_count - (MAX_ROWS / 2);
      if( $obs_count > (MAX_ROWS / 2) )
        $this->num_rows_truncated += $obs_count - (MAX_ROWS / 2);
    }
  }
  
  public function GetObservations(){ return $this->observations; }
  
  // PRIVATE
      
  private $parameters;
  private $results;
  private $sql;
  private $cone_string;
  private $database;
  private $num_rows_truncated;
  
  /// This function takes an array of tuples from the database (observation or s70 tables)
  /// and adds an entry in the observations array for each of them, as an instance of
  /// ObservationHeader.
  
  private function ExtractResultSet( $result_set )
  {
    foreach( $result_set as $next_row )
    {
      $new_obs = new ObservationHeader();
      $new_obs->FromRecord( $next_row );
      $this->observations[] = $new_obs;
    }
  }
  
  /// This function builds the conesearch calculate part of the sql query and stores it
  /// in $this->cone_string as it's used in both the select and where parts of the queries.
  
  private function BuildConeString()
  {
    $psr_rajd = $this->parameters->GetCoordinates()->GetRA()->ToDegrees();
    $psr_decjd = $this->parameters->GetCoordinates()->GetDEC()->ToDegrees();
    
    $this->cone_string = "degrees( acos(cos(radians(rajd-$psr_rajd)) *
					cos(radians(decjd)) *
					cos(radians($psr_decjd)) +
					sin(radians(decjd)) *
					sin(radians($psr_decjd))
                                        )
                                 )";
  }
  
  
  
  /// Build the basic select part of the query and add it to $this->sql
  
  private function BuildSelectString()
  {
    $this->sql .= ' SELECT *,' . $this->cone_string . ' AS distance ';
  }
  
  /// Build the from part of the query and add it to $this->sql
  private function BuildFromString( $tablename )
  {
    $this->sql .= " FROM $tablename ";
  }
  
  
  
  /// Build the where clause for the query and add it to $this->sql. This includes
  /// the conesearch part, the maximum date range and matching project_ids for any
  /// that the user might have access to. We also limit our results in order to
  /// reduce memory usage (only 16M per script) and set an order for the results.
  
  private function BuildWhereString()
  {
    $this->sql .= ' WHERE 1=1 ';

    $radius = $this->parameters->GetSearchRadius();
    
    if( $radius == '' )
      $radius = 1;
    
    $this->sql .= " AND ( (($this->cone_string - 0.25*(bmaj+bmin)) < ($radius/60.)) ) ";
    
    $project_id = $this->parameters->GetProjectID();
    if( $project_id != '')
      $this->sql .= " AND project_id = '$project_id' ";
    
    $bands = $this->parameters->GetBands();
    $nbands = count( $bands );
    if( $nbands > 0 )
    {
      $this->sql .= ' AND (';
      for( $bind = 0; $bind < $nbands; $bind ++ ) 
      {
        $this->sql .= ' ( ';
        $this->sql .= ' obsfreq >= ' . $bands[$bind]->GetLowerBound();
        $this->sql .= ' AND obsfreq <= ' . $bands[$bind]->GetUpperBound();
        $this->sql .= ' ) ';
        if( $bind < $nbands - 1 )
          $this->sql .= ' OR ';
      }
      $this->sql .= ' ) ';
    }
    
    $date_range = $this->parameters->GetDateRange();
    $from_mjd = new MJD();
    $from_mjd->FromDate( $date_range->GetFromDate() );
    $to_mjd = new MJD();
    $to_mjd->FromDate( $date_range->GetToDate() );
    $this->sql .= ' AND ( MJD <= ' . ceil( $to_mjd->ToString() );
    $this->sql .= ' OR ';
    $this->BuildAuthorizedProjectsString();
    $this->sql .= ' ) ';
    
    if( $this->parameters->GetDataType() != '' )
      $this->sql .= ' AND data_type = "' . $this->parameters->GetDataType() . '" ';
  }
  
  
  
  /// Add a simply limit clause to the $this->sql query string.
  
  private function BuildLimitString( $limit )
  {
    $this->sql .= " LIMIT $limit";
  }
  
  
  
  /// Add a simply order string to the $this->sql query string.
  
  private function BuildOrderString()
  {
    $this->sql .= ' ORDER BY ' . $this->parameters->GetOrderColumn() .
	          ' ' . $this->parameters->GetOrderDirection();
  }
  
  
  /// This function uses the list of projects that the user is authorized to see to
  /// create an IN clause that will allow them to see any observations for these
  /// projects even if they are within the proprietary period.
  
  private function BuildAuthorizedProjectsString()
  {
    $aps = $this->parameters->GetAuthorizedProjects();
    $naps = count( $aps );
    
    if( $naps == 0 )
      $this->sql .= ' 1=0 ';
    else
    {
      $this->sql .= ' project_id IN (';
      for( $ai = 0; $ai < $naps; $ai ++ )
      {
        $this->sql .= "'" . $authorized_projects[$ai] . "'";
        if( $ai < $naps - 1 )
          $this->sql .= ', ';
      }
      $this->sql .= ' ) ';
    }
  }
  
  
  /// This function counts how many observations we will get given the query parameters
  /// without actually fetching those results, so we can figure out how many we are
  /// truncating.
  
  private function CountObservationResults()
  {
    $this->sql = ' SELECT count(*) AS TRUNC_COUNT ';
    $this->BuildFromString( 'observations' );
    $this->BuildWhereString();
    $this->database->DoQuery( $this->sql );
    $only_row = $this->database->FetchRowAsArray();
    return $only_row['TRUNC_COUNT'];
  }
  
  /// This function counts how many s70 results we will get given the query parameters
  /// without actaullyl fetching those results.
  
  private function CountS70Results()
  {
    $this->sql = ' SELECT count(*) AS TRUNC_COUNT ';
    $this->BuildFromString( 's70' );
    $this->BuildWhereString();
    $this->database->DoQuery( $this->sql );
    $only_row = $this->database->GetRowAsArray();
    return $only_row['TRUNC_COUNT'];
  }
  
  
  
  /// This function creates a query string 
  
  private function GetObservationHeaders( $table, $limit )
  {
    $this->sql = '';
    $this->BuildConeString();
    $this->BuildSelectString();
    $this->BuildFromString( $table );
    $this->BuildWhereString();
    $this->BuildOrderString();
    $this->BuildLimitString( $limit );
    $this->database->DoQuery( $this->sql );
    $this->ExtractResultSet( $this->database->GetLastResult() );
  }
};




?>